﻿using BPA.Message;
using BPA.Message.Enum;
using BPASmartClient.Helper;
using BPASmartClient.Message;
using BPASmartClient.Peripheral;
using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace BPASmartClient.Device
{
    /// <summary>
    /// 设备基类
    /// </summary>
    public abstract class BaseDevice : IDevice
    {
        public BaseDevice()
        {
        }

        /// <summary>
        /// 订单物料信息
        /// </summary>
        protected OrderMaterialDelivery orderMaterialDelivery { get; set; } = new OrderMaterialDelivery();

        /// <summary>
        /// 配方数据信息
        /// </summary>
        protected RecipeBoms recipeBoms { get; set; } = new RecipeBoms();

        /// <summary>
        /// 设备ID
        /// </summary>
        public int DeviceId { get; set; }

        /// <summary>
        /// 设备所有状态
        /// </summary>
        public DeviceStatus Status { get; set; } = new DeviceStatus();

        /// <summary>
        /// 设备名称
        /// </summary>
        public string Name { get; set; }

        /// <summary>
        /// 设备类型
        /// </summary>
        public abstract DeviceClientType DeviceType { get; }

        /// <summary>
        /// 是否忙碌
        /// </summary>
        public bool IsBusy { get; protected set; }

        /// <summary>
        /// 是否健康
        /// </summary>
        public bool IsHealth { get; protected set; }

        /// <summary>
        /// 设备运行日志
        /// </summary>
        public ConcurrentDictionary<string, object> Log { get; set; }=new ConcurrentDictionary<string, object>();   
        /// <summary>
        /// 设备运行告警与错误
        /// </summary>
        public ConcurrentDictionary<string, object> Error { get; set; } = new ConcurrentDictionary<string, object>();

        public List<VariableMonitor> variableMonitors { get; set; } = new List<VariableMonitor>();

        /// <summary>
        /// 外设状态,硬件设备数据
        /// </summary>
        protected ConcurrentDictionary<string, object> peripheralStatus = new ConcurrentDictionary<string, object>();

        private List<IPeripheral> peripherals;

        /// <summary>
        /// 设备过程日志显示
        /// </summary>
        /// <param name="info"></param>
        public void DeviceProcessLogShow(string info)
        {
            DeviceProcessLog.Add(info);
            MessageLog.GetInstance.DeviceProcessLogShow(DeviceId.ToString(), info);
        }

        public void Initliaze()
        {
        }

        public void Initliaze(List<IPeripheral> peripherals)
        {
            peripherals.ForEach(p =>
            {
                p.DeviceId = this.DeviceId;
                p.Init();

            });
            this.peripherals = peripherals;
        }

        public virtual void StartMain()
        {
            #region 测试添加几个日志与告警
            Log.TryAdd(Guid.NewGuid().ToString(), new {Time=DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss"),Type="运行",Text="正常启动" });
            Log.TryAdd(Guid.NewGuid().ToString(), new { Time = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss"), Type = "打料", Text = "开始打料" });

            Error.TryAdd(Guid.NewGuid().ToString(), new { Time = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss"), Type = "落碗", Text = "落碗异常" });
            Error.TryAdd(Guid.NewGuid().ToString(), new { Time = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss"), Type = "煮面机", Text = "煮面机温度异常" });


            #endregion
            ThreadManage.GetInstance().StartLong(new Action(() =>
            {
                foreach (var peripheral in peripherals)
                {
                    string TypeName = peripheral.GetType().FullName.Replace("BPASmartClient.", "");
                    Status.Update($"{TypeName}.IsConnected", peripheral.IsConnected);
                    Status.Update($"{TypeName}.IsWork", peripheral.IsWork);
                    foreach (var key in peripheral.GetAllStatus().Keys)
                    {
                        peripheralStatus[key] = peripheral.GetAllStatus()[key];
                        Status.Update($"{TypeName}.{key}", peripheral.GetAllStatus()[key]);
                    }
                }
                Thread.Sleep(100);
            }), $"GetAllStatus:{DeviceId}");
            DoMain();
            GetGvlStatus();
        }

        private void GetGvlStatus()
        {
            this.GetType().GetFields(BindingFlags.Instance | BindingFlags.Public | BindingFlags.NonPublic).ToList().ForEach(item =>
            {
                var res = item.FieldType.GetInterfaces();
                if (res != null)
                {
                    foreach (var faces in res)
                    {
                        if (faces.Name == "IStatus")
                        {
                            IStatus status = item.GetValue(this) as IStatus;
                            GetMonitorData(status);
                            ThreadManage.GetInstance().StopTask($"{item.Name}:{DeviceId}", new Action(() =>
                             {
                                 ThreadManage.GetInstance().StartLong(new Action(() =>
                                 {
                                     UpdateValue(status);
                                     Thread.Sleep(1000);
                                 }), $"{item.Name}:{DeviceId}");
                             }));
                        }
                    }
                }
            });
        }

        /// <summary>
        /// 获取流程信息
        /// </summary>
        private void GetMonitorData(IStatus status)
        {
            if (status == null) return;
            foreach (var item in status.GetType().GetProperties())
            {
                if (item.CustomAttributes.Count() > 0)
                {
                    var attributeName = item.CustomAttributes.FirstOrDefault(p => p.AttributeType.Name == "VariableMonitorAttribute");
                    if (attributeName == null) return;
                    var plcadd = item.GetCustomAttribute<VariableMonitorAttribute>()?.PLCAddress;
                    var modadd = item.GetCustomAttribute<VariableMonitorAttribute>()?.ModbusTcpAddress;
                    var notes = item.GetCustomAttribute<VariableMonitorAttribute>()?.Notes;

                    if (item.PropertyType?.BaseType?.Name == "Array")
                    {
                        if (plcadd?.Length > 0)
                        {
                            var arrayRes = item.GetValue(status, null);
                            if (arrayRes != null && arrayRes is Array arr)
                            {
                                for (int i = 0; i < arr.Length; i++)
                                {
                                    var res = variableMonitors.FirstOrDefault(p => p.VarName == $"{item.Name}_{i + 1}");
                                    if (res == null)
                                    {
                                        string[] plc = plcadd?.Substring(1).Split('.');
                                        string TempPlcAddress = string.Empty;
                                        if (plc?.Length == 2)
                                        {
                                            int add = int.Parse(plc[1]);
                                            int firstAdd = int.Parse(plc[0]);
                                            if (add >= 0 && add < 7)
                                            {
                                                add += i;
                                            }
                                            else if (add >= 7)
                                            {
                                                add = 0;
                                                firstAdd++;
                                            }
                                            plc[0] = firstAdd.ToString();
                                            plc[1] = add.ToString();
                                            TempPlcAddress = $"M{plc[0]}.{plc[1]}";
                                        }

                                        variableMonitors.Add(new VariableMonitor()
                                        {
                                            Id = variableMonitors.Count,
                                            VarName = $"{item.Name}_{i + 1}",
                                            Notes = $"{notes}_{i + 1}",
                                            ModbusTcpAddress = $"{int.Parse(modadd) + i}",
                                            PLCAddress = TempPlcAddress,

                                        });
                                    }
                                }
                            }
                        }
                        else
                        {
                            var arrayRes = item.GetValue(status, null);
                            if (arrayRes != null && arrayRes is Array arr)
                            {
                                for (int i = 0; i < arr.Length; i++)
                                {
                                    var res = variableMonitors.FirstOrDefault(p => p.VarName == $"{item.Name}_{i + 1}");
                                    if (res == null)
                                    {
                                        variableMonitors.Add(new VariableMonitor()
                                        {
                                            Id = variableMonitors.Count,
                                            VarName = $"{item.Name}_{i + 1}",
                                            Notes = $"{notes}_{i + 1}",

                                        });
                                    }
                                }
                            }
                        }
                    }
                    else
                    {
                        var res = variableMonitors.FirstOrDefault(p => p.VarName == item.Name);
                        if (res == null)
                        {
                            variableMonitors.Add(new VariableMonitor()
                            {
                                Id = variableMonitors.Count,
                                VarName = item.Name,
                                Notes = notes,
                                ModbusTcpAddress = modadd,
                                PLCAddress = plcadd,

                            });
                        }
                    }

                }
            }
        }

        public void UpdateValue(IStatus status)
        {
            if (status == null) return;
            foreach (var item in status.GetType().GetProperties())
            {
                if (item.CustomAttributes.Count() > 0)
                {
                    if (item.PropertyType?.BaseType?.Name == "Array")
                    {
                        var arrayRes = item.GetValue(status);
                        if (arrayRes != null && arrayRes is Array arr)
                        {
                            for (int i = 0; i < arr.Length; i++)
                            {
                                int index = Array.FindIndex(variableMonitors.ToArray(), p => p.VarName == $"{item.Name}_{i + 1}");
                                if (index >= 0 && index < variableMonitors.Count)
                                {
                                    //App.Current.Dispatcher.Invoke(new Action(() =>
                                    //{
                                    variableMonitors.ElementAt(index).CurrentValue = arr.GetValue(i)?.ToString();
                                    //}));

                                }
                            }
                        }
                    }
                    else
                    {
                        int index = Array.FindIndex(variableMonitors.ToArray(), p => p.VarName == item.Name);
                        if (index >= 0 && index < variableMonitors.Count)
                        {
                            //App.Current.Dispatcher.Invoke(new Action(() =>
                            //{
                            variableMonitors.ElementAt(index).CurrentValue = item.GetValue(status)?.ToString();
                            //}));
                        }
                    }

                }
            }
        }

        public abstract void DoMain();
        public abstract void Stop();
    }
}
